﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinToolbars;
using Infragistics.Win;

namespace DistinctCheckBox
{
	internal class GridDistinctCheckboxHelper
	{
		protected UltraGrid m_Grid;
		
		/// <summary>
		/// Initializes a new instance of ControlHelperBase with the given control
		/// </summary>
		/// <param name="control">The control related to this helper</param>
		public GridDistinctCheckboxHelper(UltraGrid grid)
		{
			Grid = grid;
			UniqueCheckboxColumns = new List<DistinctCheckBoxColumn>();
		}

		/// <summary>
		/// Gets or sets the control related to this helper
		/// </summary>
		public UltraGrid Grid
		{
			get { return m_Grid; }
			set 
			{
				if (m_Grid != value)
				{
					if (m_Grid != null)
						OnControlDetached(m_Grid);
					
					m_Grid = value;

					if (m_Grid != null)
						OnControlAttached(m_Grid);
				}
			}
		}

		/// <summary>
		/// A list of the grid columns which contain a distinct checkbox
		/// </summary>
		public List<DistinctCheckBoxColumn> UniqueCheckboxColumns { get; protected set; }

		/// <summary>
		/// Registers the grid's distinct checkbox event handlers
		/// </summary>
		/// <param name="grid">The grid which includes distinct checkboxes</param>
		protected void OnControlAttached(UltraGrid grid)
		{
			grid.BeforeCellUpdate += new BeforeCellUpdateEventHandler(Grid_BeforeCellUpdate);
			grid.CellChange += new CellEventHandler(grid_AfterCellUpdate);
		}

		/// <summary>
		/// Unregisters the grid's distinct checkbox event handlers
		/// </summary>
		/// <param name="grid">The grid which includes distinct checkboxes</param>
		protected void OnControlDetached(UltraGrid grid)
		{
			grid.BeforeCellUpdate -= new BeforeCellUpdateEventHandler(Grid_BeforeCellUpdate);
			grid.CellChange -= new CellEventHandler(grid_AfterCellUpdate);
		}

		/// <summary>
		/// Prevents clearing the last cell checked if one cell must be checked
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void Grid_BeforeCellUpdate(object sender, BeforeCellUpdateEventArgs e)
		{
			if (e.Cell.StyleResolved != ColumnStyle.CheckBox ||
				!IsColumnDistinctCheckBox(e.Cell.Column) ||
				IsNoCellCheckedAllowed(e.Cell.Column))
				return;

			if ((bool)e.NewValue == false && !IsBandContainsCheckedCell(e.Cell))
				e.Cancel = true;
		}

		/// <summary>
		///
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void grid_AfterCellUpdate(object sender, CellEventArgs e)
		{
			if (e.Cell.StyleResolved == Infragistics.Win.UltraWinGrid.ColumnStyle.CheckBox)
				Grid.PerformAction(UltraGridAction.DeactivateCell);

			if (e.Cell.StyleResolved != ColumnStyle.CheckBox ||
				(!IsColumnDistinctCheckBox(e.Cell.Column)))
				return;

			ClearPreviouslyCheckedCells(e.Cell);
		}
		
		/// <summary>
		/// Returns whether a column is considered a distinct check box column
		/// </summary>
		/// <param name="column">The column to check</param>
		/// <returns>True if the column is considered a distinct check box column; otherwise, false</returns>
		protected bool IsColumnDistinctCheckBox(UltraGridColumn column)
		{
			return UniqueCheckboxColumns.FirstOrDefault(c => Object.ReferenceEquals(c.CheckBoxColumn, column)) != null;
		}

		/// <summary>
		/// Returns whether unchecking all column cells is allowed
		/// </summary>
		/// <param name="column">The column to check</param>
		/// <returns>True if unchecking all column cells is allowed; otherwise, false</returns>
		protected bool IsNoCellCheckedAllowed(UltraGridColumn column)
		{
			return UniqueCheckboxColumns.FirstOrDefault(c => Object.ReferenceEquals(c.CheckBoxColumn, column) && c.AllowNoCellChecked) != null;
		}

		/// <summary>
		/// Returns whether band rows contains another checked cell and both rows contains the same values in the group by columns
		/// </summary>
		/// <param name="cell">The checked cell</param>
		/// <returns>True if band rows contains another checked cell and both rows contains the same values in the group by columns; otherwise, false</returns>
		private bool IsBandContainsCheckedCell(UltraGridCell checkedCell)
		{
			var bandRows = Grid.Rows.GetRowEnumerator(GridRowType.DataRow, checkedCell.Column.Band, null).OfType<UltraGridRow>();

			List<UltraGridColumn> groupByColumns;

			foreach (var row in bandRows)
			{
				if (row != checkedCell.Row && Convert.ToBoolean(row.GetCellValue(checkedCell.Column)))
				{
					groupByColumns = UniqueCheckboxColumns.FirstOrDefault(c => object.ReferenceEquals(c.CheckBoxColumn, checkedCell.Column)).GroupByColumns;

					if (groupByColumns.All(g => row.GetCellValue(g).Equals(checkedCell.Row.GetCellValue(g))))
						return true;
				}					
			}

			return false;
		}

		/// <summary>
		/// Clears all other checkboxes which are checked in the same grid column if the GroupByColumns are identical
		/// </summary>
		/// <param name="cell">The cell which is currently checked</param>
		private void ClearPreviouslyCheckedCells(UltraGridCell checkedCell)
		{
			var bandRows = Grid.Rows.GetRowEnumerator(GridRowType.DataRow, checkedCell.Column.Band, null).Cast<UltraGridRow>();

			Grid.EventManager.SetEnabled(GridEventIds.BeforeCellUpdate, false);
			Grid.EventManager.SetEnabled(GridEventIds.AfterCellUpdate, false);

			List<UltraGridColumn> groupByColumns;

			foreach (var row in bandRows)
			{
				if (row != checkedCell.Row)
				{
					groupByColumns = UniqueCheckboxColumns.FirstOrDefault(c => object.ReferenceEquals(c.CheckBoxColumn, checkedCell.Column)).GroupByColumns;

					if (groupByColumns.All(g => row.GetCellValue(g).Equals(checkedCell.Row.GetCellValue(g))))
						row.Cells[checkedCell.Column].Value = false;
				}
			}

			Grid.EventManager.SetEnabled(GridEventIds.BeforeCellUpdate, true);
			Grid.EventManager.SetEnabled(GridEventIds.AfterCellUpdate, true);
		}
	}
}
